﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stormtrooper
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetStormtroopersGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;

        public int TecCount = 0; // текущее количество объектов в массиве
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetStormtroopersGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T stormtrooper)
        {
            return Insert(stormtrooper, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T stormtrooper, int position)
        {

            if (position < 0 || position >= Count || TecCount == Count) return -1;

            TecCount++;
            while (_places[position] != null)
            {
                for (int i = _places.Length - 1; i > 0; --i)
                {
                    if (_places[i] == null && _places[i - 1] != null)
                    {
                        _places[i] = _places[i - 1];
                        _places[i - 1] = null;
                    }
                }
            }
            _places[position] = stormtrooper;
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= _places.Length) return null;
            T saveStorm = _places[position];
            _places[position] = null;
            return saveStorm;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            if (position < 0 || position >= Count || _places[position] == null) return null;
            return _places[position];
        }
    }
}
